#!/bin/python

import os, os.path, shutil, struct
import glob
import sys

def read_int(fin,fout=None):
	data = fin.read(4)
	s, = struct.unpack('i',data)
	if fout:
		fout.write(data)
	return s

def read_size_t(fin,fout=None):
	data = fin.read(8)
	s, = struct.unpack('Q',data)
	if fout:
		fout.write(data)
	return s

def read_string(fin,fout=None):
	s_len = read_size_t(fin,fout)
	if (s_len > 9999999):
		raise RuntimeError("string too long: ",s_len)
	s = fin.read(s_len)
	if fout:
		fout.write(s)
	return s

def read_tmap(fin,fout=None):
	#will be 1
	t_map = read_size_t(fin,fout)
	#print 't_map_size: ',t_map

	for j in range(t_map):
		#print 't_map: ',j

		key = read_string(fin,fout)
		#print 'key: ',key

		val_size = read_size_t(fin,fout)
		#print 'val size: ',val_size

		for v in range(val_size):
			key = read_string(fin,fout)
			#print 'key: ',key
			val = read_string(fin,fout)
			#print 'val: ',val

def write_empty_map(fout):
	val = struct.pack('Q',0)
	fout.write(val)

def parse_type_file(fin,fout=None):
	map_size = read_size_t(fin,fout)

	#print 'map_size: ',map_size

	#map of type_id (string) to type
	for i in range(map_size):
		#print '\nmap: ',i
		#key
		type_id = read_string(fin,fout)
		#print 'type_id: ',type_id

		#value is encoded type
		
		#type id
		type_id2 = read_string(fin,fout)
		#print 'type_id2: ',type_id

		#schema name
		schema_name = read_string(fin,fout)
		#print 'schema name: ',schema_name

		#attributes
		num_attrs = read_size_t(fin,fout)
		#print 'num_attrs: ',num_attrs

		for j in range(num_attrs):
			#print 'attr: ',j

			field_name = read_string(fin,fout)
			#print 'field name: ',field_name

			field_type = read_string(fin,fout)
			#print 'field type: ',field_type

			field_annotation = read_int(fin,fout)
			#print 'field annotations: ',field_annotation

		sec_attr = read_string(fin,fout)
		#print 'sec_attr: ',sec_attr

		read_tmap(fin,fout);

		label = read_string(fin,fout)
		#print 'label: ',label

		semantic_type = read_string(fin,fout)
		#print 'semantic_type: ',semantic_type

		#params map; might need to insert
		if (fout is None):
			read_tmap(fin)
		else:
			write_empty_map(fout)



def fix_type_file(fname):

	fin = open(fname,"rb")
	need_to_update = False
	try:
		parse_type_file(fin)
		need_to_update = False
	except (RuntimeError,MemoryError):
		#print '\terror parsing type file %s; we probably have to update it: %s'%(fname,sys.exc_info()[0])
		need_to_update = True
	except:
		#print 'unknown error: ',sys.exc_info()[0]
		need_to_update = True
	fin.close()

	if (need_to_update):
		print 'updating file: ',fname
		bkup_file = fname+'.bkup'
		shutil.move(fname,bkup_file)

		fin = open(bkup_file,"rb")
		fout = open(fname,"wb")

		parse_type_file(fin,fout)

		fin.close()
		fout.close()
	else:
		print 'file: %s is okay'%(fname)

if len(sys.argv) !=2:
	print 'need to specify the path to the persist files; i.e. %s /opt/gpudb/persist'%(sys.argv[0])
	sys.exit()

fdir = sys.argv[1]

#rank0
fname = os.path.join(fdir,"rank0.sm/rank0.sm.types")

if (os.path.exists(fname)):
	fix_type_file(fname)


#other ranks
all_type_files = glob.glob(os.path.join(fdir,'rank*[0-9]-tom*[0-9]/rank*[0-9]-tom*[0-9].tom.types'))
for fname in all_type_files:
	fix_type_file(fname)


print 'done'